@@ -1,5 +1,6 @@
 import time
 from datetime import datetime
+from urlparse import urlparse
 import datetime as datetime_mod
 import os
 import sys
@@ -11,7 +12,7 @@ except:
     import simplejson as json
 
 
-class IronTokenProvider:
+class IronTokenProvider(object):
     def __init__(self, token):
         self.token = token
 
@@ -19,22 +20,18 @@ class IronTokenProvider:
         return self.token
 
 
-class KeystoneTokenProvider:
+class KeystoneTokenProvider(object):
     def __init__(self, keystone):
         self.server = keystone["server"] + ("" if keystone["server"].endswith("/") else "/")
         self.tenant = keystone["tenant"]
         self.username = keystone["username"]
         self.password = keystone["password"]
         self.token = None
-        self.local_expires_at = None
+        self.local_expires_at_timestamp = 0
 
 
     def getToken(self):
-        if self.local_expires_at:
-            date_diff = time.mktime(datetime.now().timetuple()) - time.mktime(self.local_expires_at.timetuple())
-        else:
-            date_diff = time.mktime(datetime.now().timetuple())
-
+        date_diff = time.mktime(datetime.now().timetuple()) - self.local_expires_at_timestamp
         if self.token is None or date_diff > -10:
             payload = {
                 'auth': {
@@ -56,18 +53,18 @@ class KeystoneTokenProvider:
             expires = dateutil.parser.parse(token_data['expires']).replace(tzinfo=None)
             duration = expires - issued_at
 
-            self.local_expires_at = datetime.now() + duration
+            self.local_expires_at_timestamp = time.mktime((datetime.now() + duration).timetuple())
             self.token = token_data['id']
 
         return self.token
 
 
-class IronClient:
+class IronClient(object):
     __version__ = "1.1.2"
 
     def __init__(self, name, version, product, host=None, project_id=None,
                  token=None, protocol=None, port=None, api_version=None,
-                 config_file=None, keystone=None):
+                 config_file=None, keystone=None, cloud=None, path_prefix=None):
         """Prepare a Client that can make HTTP calls and return it.
 
         Keyword arguments:
@@ -96,6 +93,8 @@ class IronClient:
                 "project_id": None,
                 "token": None,
                 "keystone": None,
+                "path_prefix": None,
+                "cloud": None,
         }
         products = {
                 "iron_worker": {
@@ -123,7 +122,7 @@ class IronClient:
         config = configFromFile(config, config_file, product)
         config = configFromArgs(config, host=host, project_id=project_id,
                 token=token, protocol=protocol, port=port,
-                api_version=api_version, keystone=keystone)
+                api_version=api_version, keystone=keystone, cloud=cloud)
 
         required_fields = ["project_id"]
 
@@ -157,16 +156,26 @@ class IronClient:
         self.protocol = config["protocol"]
         self.port = config["port"]
         self.api_version = config["api_version"]
+        self.cloud = config["cloud"]
         self.conn = requests.Session()
         self.headers = {
                 "Accept": "application/json",
                 "User-Agent": "%s (version: %s)" % (self.name, self.version)
         }
+
+        if self.cloud is not None:
+            url = urlparse(self.cloud)
+            self.protocol = url.scheme
+            self.host = url.netloc.split(":")[0]
+            if url.port:
+                self.port = url.port
+            self.path_prefix = url.path.rstrip("/")
+
         if self.protocol == "https" and self.port == 443:
-            self.base_url = "%s://%s/%s/" % (self.protocol, self.host, self.api_version)
+            self.base_url = "%s://%s%s/%s/" % (self.protocol, self.host, self.path_prefix, self.api_version)
         else:
-            self.base_url = "%s://%s:%s/%s/" % (self.protocol, self.host,
-                                                self.port, self.api_version)
+            self.base_url = "%s://%s:%s%s/%s/" % (self.protocol, self.host,
+                                                self.port, self.path_prefix, self.api_version)
         if self.project_id:
             self.base_url += "projects/%s/" % self.project_id
         if self.protocol == "https" and self.port != 443:
@@ -176,6 +185,7 @@ class IronClient:
     def _doRequest(self, url, method, body="", headers={}):
         if self.token or self.keystone:
             headers["Authorization"] = "OAuth %s" % self.token_provider.getToken()
+
         if method == "GET":
             r = self.conn.get(url, headers=headers)
         elif method == "POST":
@@ -397,4 +407,4 @@ def configFromArgs(config, **kwargs):
     return config
 
 def intersect(a, b):
-    return list(set(a) & set(b))
\ No newline at end of file
+    return list(set(a) & set(b))
